@@ -1011,6 +1011,27 @@ public class MainWindow {
 		}
 	}
 
+#if false
+	public void ImportUdi (string udi)
+	{
+		/* probably a camera we need to contruct on of our gphoto2 uris */
+		Hal.Device dev = new Hal.Device (Core.HalContext, udi);
+		string mount_point = dev.GetPropertyString ("volume.mount_point");
+		int bus = dev.GetPropertyInt ("usb.bus_number");
+		int device = dev.GetPropertyInt ("usb.linux.device_number");
+		System.Console.WriteLine ("dev = {1} exists = {2} mount_point = {0} {3},{4}", mount_point, dev, dev.Exists, bus, device);
+
+		if (! dev.Exists || mount_point != null) {
+			ImportFile (mount_point);
+		} else {
+			string gphoto_uri = String.Format ("gphoto2:usb:{0},{1}", bus.ToString ("d3") , device.ToString ("d3"));
+			System.Console.WriteLine ("gphoto_uri = {0}", gphoto_uri);
+			ImportCamera (gphoto_uri);
+		} 
+			
+	}
+#endif
+
 	void HandleIconViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
 	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
@@ -1186,7 +1207,7 @@ public class MainWindow {
 	{
 		db.Sync = false;
 		ImportCommand command = new ImportCommand (main_window);
-		if (command.ImportFromFile (db.Photos, this.last_import_path) > 0) {
+		if (command.ImportFromFile (db.Photos, null) > 0) {
 			this.last_import_path = command.ImportPath;
 			UpdateQuery ();
 		}
@@ -1360,7 +1381,7 @@ public class MainWindow {
 		GnomeUtil.UrlShow (main_window, url.ToString ());
 	}
 
-	void HandleAbout (object sender, EventArgs args)
+	public static void HandleAbout (object sender, EventArgs args)
 	{
 		string [] authors = new string [] {
 			"Ettore Perazzoli",
@@ -1406,7 +1427,7 @@ public class MainWindow {
 
                 new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
-			   "Copyright 2003-2005 Novell Inc.",
+			   Mono.Posix.Catalog.GetString ("Copyright 2003-2006 Novell Inc."),
                            null, authors, new string [0], translators, null).Show();
 	}
 
@@ -2383,6 +2404,43 @@ public class MainWindow {
 		}
 	}
 
+	public void HandleOpenWith (Gnome.Vfs.MimeApplication mime_application)
+	{
+		Photo[] selected = SelectedPhotos ();
+
+		if (selected == null || selected.Length < 1)
+			return;
+		
+		string header = Catalog.GetPluralString ("Create New Version?", "Create New Versions?", selected.Length); 
+		
+		string msg = String.Format (Catalog.GetPluralString (
+				"Before launching {1}, should F-Spot create a new version of the selected photo to preserve the original?",
+				"Before launching {1}, should F-Spot create new versions of the selected photos to preserve the originals?", selected.Length),
+				selected.Length, mime_application.Name);
+
+		// FIXME add a cancel button?
+		Gtk.ResponseType response = HigMessageDialog.RunHigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+									   MessageType.Question, Gtk.ButtonsType.YesNo, 
+									   header, msg);
+
+		bool create_new_versions = (response == Gtk.ResponseType.Yes);
+
+		GLib.List uri_list = new GLib.List (typeof (string));
+		foreach (Photo photo in selected) {
+			if (create_new_versions) {
+				// exception handling? Out of space, blah, blah
+				// FIXME give the new version a better name, based on the app's name that we're opening it with
+				uint version = photo.CreateNamedVersion (mime_application.Name, photo.DefaultVersionId, true);
+				photo.DefaultVersionId = version;
+				UpdateForVersionIdChange (version);
+			}
+
+			uri_list.Append (photo.DefaultVersionPath);
+		}
+		
+		mime_application.Launch (uri_list);
+	}
+
 	// Tag typing
 
 	private ArrayList selected_photos_tagnames;
@@ -2657,6 +2715,20 @@ public class MainWindow {
 		return false;
 	}
 
+	public string [] SelectedMimeTypes ()
+	{
+		ArrayList mimes = new ArrayList ();
+
+		foreach (Photo p in SelectedPhotos ()) {
+			string mime = Gnome.Vfs.MimeType.GetMimeTypeForUri (p.DefaultVersionPath);
+
+			if (! mimes.Contains (mime))
+				mimes.Add (mime);
+		}
+
+		return mimes.ToArray (typeof (string)) as string [];
+	}
+
 	public static void SetTip (Widget widget, string tip)
 	{
 		toolTips.SetTip (widget, tip, null);
