@@ -123,6 +123,9 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		get {
 			return time;
 		}
+		set {
+			time = value;
+		}
 	}
 
 	private string directory_path;
@@ -343,6 +346,26 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		}
 	}
 
+	public uint CreateNamedVersion (string name, uint base_version_id, bool create_file)
+	{
+		int num = 1;
+		
+		string final_name;
+		while (true) {
+			final_name = String.Format (
+					Mono.Posix.Catalog.GetPluralString ("Modified in {1}", "Modified in {1} ({0})", num),
+					num, name);
+
+			if (num > 1)
+				final_name = name + String.Format(" ({0})", num);
+
+			if (! VersionNameExists (final_name))
+				return CreateVersion (final_name, base_version_id, create_file);
+
+			num ++;
+		}
+	}
+
 	public void RenameVersion (uint version_id, string new_name)
 	{
 		if (version_id == OriginalVersionId)
@@ -426,9 +449,10 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		return tags.Contains (tag);
 	}
 
-	private static FSpot.Xmp.XmpFile CreateXmp (FSpot.IBrowsableItem item)
+	private static FSpot.Xmp.XmpFile UpdateXmp (FSpot.IBrowsableItem item, FSpot.Xmp.XmpFile xmp)
 	{
-		FSpot.Xmp.XmpFile xmp = new FSpot.Xmp.XmpFile ();
+		if (xmp == null) 
+			xmp = new FSpot.Xmp.XmpFile ();
 
 		Tag [] tags = item.Tags;
 		string [] names = new string [tags.Length];
@@ -436,7 +460,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		for (int i = 0; i < tags.Length; i++)
 			names [i] = tags [i].Name;
 		
-		FSpot.MetadataStore.Add (xmp, "dc:subject", "rdf:Bag", names);
+		xmp.Store.Update ("dc:subject", "rdf:Bag", names);
 		xmp.Dump ();
 
 		return xmp;
@@ -451,11 +475,19 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			FSpot.JpegFile jimg = img as FSpot.JpegFile;
 			
 			jimg.SetDescription (this.Description);
-		
-			FSpot.Xmp.XmpFile xmp = CreateXmp (this);
-			jimg.SetXmp (xmp);
+			jimg.SetDateTimeOriginal (this.Time);
+			jimg.SetXmp (UpdateXmp (this, jimg.Header.GetXmp ()));
 
 			jimg.SaveMetaData (path);
+		} else if (img is FSpot.Png.PngFile) {
+			FSpot.Png.PngFile png = img as FSpot.Png.PngFile;
+			
+			if (img.Description != this.Description)
+				png.SetDescription (this.Description);
+			
+			png.SetXmp (UpdateXmp (this, png.GetXmp ()));
+
+			png.Save (path);
 		}
 	}
 
@@ -463,22 +495,26 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	{
 		uint version = DefaultVersionId;
 		ImageFile img = ImageFile.Create (DefaultVersionUri.LocalPath);
-		Exif.ExifData exif_data = new Exif.ExifData (DefaultVersionPath);
 
 		// Always create a version if the source is not a jpeg for now.
 		create_version = create_version || !(img is FSpot.JpegFile);
 
+		if (buffer == null)
+			throw new ApplicationException ("invalid (null) image");
+
 		if (create_version)
 			version = CreateDefaultModifiedVersion (DefaultVersionId, false);
 
 		try {
 			string version_path = GetVersionPath (version);
 			
-			PixbufUtils.SaveJpeg (buffer, version_path, 95, exif_data);
+			using (Stream stream = File.OpenWrite (version_path)) {
+				img.Save (buffer, stream);
+			}
 			FSpot.ThumbnailGenerator.Create (version_path).Dispose ();
 			DefaultVersionId = version;
-			
 		} catch (System.Exception e) {
+			System.Console.WriteLine (e);
 			if (create_version)
 				DeleteVersion (version);
 			
@@ -489,7 +525,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	}
 
 	// Constructor
-	public Photo (uint id, uint unix_time, string directory_path, string name)
+	public Photo (uint id, long unix_time, string directory_path, string name)
 		: base (id)
 	{
 		time = DbUtils.DateTimeFromUnixTime (unix_time);
@@ -504,7 +540,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		AddVersionUnsafely (OriginalVersionId, Mono.Posix.Catalog.GetString ("Original"));
 	}
 
-	public Photo (uint id, uint unix_time, string path)
+	public Photo (uint id, long unix_time, string path)
 		: this (id, unix_time,
 
 			System.IO.Path.GetDirectoryName (path),
@@ -643,7 +679,7 @@ public class PhotoStore : DbStore {
 	public Photo Create (string newPath, string origPath, out Pixbuf thumbnail)
 	{
 		FSpot.ImageFile img = FSpot.ImageFile.Create (origPath);
-		uint unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
+		long unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
 		string description = img.Description != null ? img.Description : "";
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
@@ -857,7 +893,7 @@ public class PhotoStore : DbStore {
 
 		if (reader.Read ()) {
 			photo = new Photo (id,
-					   Convert.ToUInt32 (reader [0]),
+					   Convert.ToInt64 (reader [0]),
 					   reader [1].ToString (),
 					   reader [2].ToString ());
 
@@ -905,7 +941,7 @@ public class PhotoStore : DbStore {
 
 		if (reader.Read ()) {
 			photo = new Photo (Convert.ToUInt32 (reader [0]),
-					   Convert.ToUInt32 (reader [1]),
+					   Convert.ToInt64 (reader [1]),
 					   directory_path,
 					   filename);
 
@@ -988,17 +1024,36 @@ public class PhotoStore : DbStore {
 
 	public override void Commit (DbItem item)
 	{
-		Photo photo = item as Photo;
+		DbItemEventArgs args = new DbItemEventArgs (item);
+		Commit (args.Items, args);
+	}
 
+	public void Commit (DbItem [] items, DbItemEventArgs args)
+	{
+		if (items.Length > 1)
+			BeginTransaction ();
+
+		foreach (DbItem item in items) {
+			Update ((Photo)item);
+		}
+		EmitChanged (items, args);
+
+		if (items.Length > 1)
+			CommitTransaction ();
+	}
+	
+        private void Update (Photo photo) {
 		// Update photo.
 
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 		command.CommandText = String.Format ("UPDATE photos SET description = '{0}',     " +
-						     "                  default_version_id = {1} " +
-						     "              WHERE id = {2}",
+						     "                  default_version_id = {1}, " +
+						     "                  time = {2} " +
+						     "              WHERE id = {3}",
 						     SqlString (photo.Description),
 						     photo.DefaultVersionId,
+						     DbUtils.UnixTimeFromDateTime (photo.Time),
 						     photo.Id);
 		command.ExecuteNonQuery ();
 		command.Dispose ();
@@ -1043,8 +1098,6 @@ public class PhotoStore : DbStore {
 			command.ExecuteNonQuery ();
 			command.Dispose ();
 		}
-
-		EmitChanged (item);
 	}
 	
 	public class DateRange 
@@ -1082,15 +1135,11 @@ public class PhotoStore : DbStore {
 
 	public Photo [] Query (string query)
 	{
-		//Console.WriteLine ("Query Start {0}", System.DateTime.Now.ToLongTimeString ());
-
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 		command.CommandText = query;
 		SqliteDataReader reader = command.ExecuteReader ();
 		
-		//Console.WriteLine ("Query Mid {0}", System.DateTime.Now);
-
 		ArrayList version_list = new ArrayList ();
 		ArrayList id_list = new ArrayList ();
 		while (reader.Read ()) {
@@ -1099,7 +1148,7 @@ public class PhotoStore : DbStore {
 
 			if (photo == null) {
 				photo = new Photo (id,
-						   Convert.ToUInt32 (reader [1]),
+						   Convert.ToInt64 (reader [1]),
 						   reader [2].ToString (),
 						   reader [3].ToString ());
 				
@@ -1112,10 +1161,7 @@ public class PhotoStore : DbStore {
 			id_list.Add (photo);
 		}
 
-		//Console.WriteLine ("Query End {0}", System.DateTime.Now.ToLongTimeString ());
-
 		bool need_load = false;
-		//Console.WriteLine ("Start {0}", System.DateTime.Now);
 		foreach (Photo photo in version_list) {
 			AddToCache (photo);
 			need_load |= !photo.Loaded;
@@ -1131,7 +1177,6 @@ public class PhotoStore : DbStore {
 			//Console.WriteLine ("Skipped Loading Data");
 		}
 
-		//Console.WriteLine ("End {0}", System.DateTime.Now);
 		command.Dispose ();
 
 		return id_list.ToArray (typeof (Photo)) as Photo [];
@@ -1151,13 +1196,20 @@ public class PhotoStore : DbStore {
 		return Query (query_string);
 	}
 
-	public Photo [] QueryUntagged ()
+	public Photo [] QueryUntagged (DateRange range)
 	{
 		StringBuilder query_builder = new StringBuilder ();
 
 		query_builder.Append ("SELECT * FROM photos WHERE id NOT IN " +
-					"(SELECT DISTINCT photo_id FROM photo_tags) " +
-					"ORDER BY time");
+					"(SELECT DISTINCT photo_id FROM photo_tags) ");
+		
+		if (range != null) {
+			query_builder.Append (String.Format ("AND photos.time >= {0} AND photos.time < {1} ",
+							     DbUtils.UnixTimeFromDateTime (range.Start), 
+							     DbUtils.UnixTimeFromDateTime (range.End)));
+		}
+
+		query_builder.Append("ORDER BY time");
 
 		return Query (query_builder.ToString ());
 	}
